@@ -1109,7 +1109,7 @@ cell_error_data_func (GtkTreeViewColumn *tree_column,
 }
 
 static void
-finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_data)
+finish_setup (CEPageIP4 *self, gpointer user_data)
 {
 	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
 	GtkTreeSelection *selection;
@@ -1117,9 +1117,6 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 
-	if (error)
-		return;
-
 	populate_ui (self);
 
 	/* IP Address column */
@@ -1240,7 +1237,7 @@ ce_page_ip4_new (NMConnectionEditor *editor,
 	priv->setting = nm_connection_get_setting_ip4_config (connection);
 	g_assert (priv->setting);
 
-	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
+	g_signal_connect (self, CE_PAGE_INITIALIZED, G_CALLBACK (finish_setup), NULL);
 
 	return CE_PAGE (self);
 }
