@@ -1091,7 +1091,7 @@ cell_error_data_func (GtkTreeViewColumn *tree_column,
 }
 
 static void
-finish_setup (CEPageIP6 *self, gpointer unused, GError *error, gpointer user_data)
+finish_setup (CEPageIP6 *self, gpointer user_data)
 {
 	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
 	GtkTreeSelection *selection;
@@ -1099,9 +1099,6 @@ finish_setup (CEPageIP6 *self, gpointer unused, GError *error, gpointer user_dat
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 
-	if (error)
-		return;
-
 	populate_ui (self);
 
 	/* IP Address column */
@@ -1222,7 +1219,7 @@ ce_page_ip6_new (NMConnectionEditor *editor,
 	priv->setting = nm_connection_get_setting_ip6_config (connection);
 	g_assert (priv->setting);
 
-	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
+	g_signal_connect (self, CE_PAGE_INITIALIZED, G_CALLBACK (finish_setup), NULL);
 
 	return CE_PAGE (self);
 }
